@@ -47,20 +47,25 @@ License   : matplotlib license (PSF compatible)
             see license/LICENSE_TTFQUERY.
 """
 
-import os, sys, warnings
-try:
-    set
-except NameError:
-    from sets import Set as set
+from collections import Iterable
+import json
+import os
+import sys
+from threading import Timer
+import warnings
+
 import matplotlib
-from matplotlib import afm
-from matplotlib import ft2font
-from matplotlib import rcParams, get_cachedir
+from matplotlib import afm, cbook, ft2font, rcParams, get_cachedir
 from matplotlib.cbook import is_string_like
-import matplotlib.cbook as cbook
 from matplotlib.compat import subprocess
-from matplotlib.fontconfig_pattern import \
-    parse_fontconfig_pattern, generate_fontconfig_pattern
+from matplotlib.fontconfig_pattern import (
+    parse_fontconfig_pattern, generate_fontconfig_pattern)
+
+try:
+    from functools import lru_cache
+except ImportError:
+    from functools32 import lru_cache
+
 
 USE_FONTCONFIG = False
 verbose = matplotlib.verbose
@@ -117,11 +122,13 @@ font_family_aliases = set([
 MSFolders = \
     r'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders'
 
-MSFontDirectories   = [
+
+MSFontDirectories = [
     r'SOFTWARE\Microsoft\Windows NT\CurrentVersion\Fonts',
     r'SOFTWARE\Microsoft\Windows\CurrentVersion\Fonts']
 
-X11FontDirectories  = [
+
+X11FontDirectories = [
     # an old standard installation point
     "/usr/X11R6/lib/X11/fonts/TTF/",
     "/usr/X11/lib/X11/fonts",
@@ -142,7 +149,7 @@ OSXFontDirectories = [
     ""
 ]
 
-if not USE_FONTCONFIG:
+if not USE_FONTCONFIG and sys.platform != 'win32':
     home = os.environ.get('HOME')
     if home is not None:
         # user fonts on OSX
@@ -151,6 +158,7 @@ if not USE_FONTCONFIG:
         path = os.path.join(home, '.fonts')
         X11FontDirectories.append(path)
 
+
 def get_fontext_synonyms(fontext):
     """
     Return a list of file extensions extensions that are synonyms for
@@ -160,6 +168,7 @@ def get_fontext_synonyms(fontext):
             'otf': ('ttf', 'otf'),
             'afm': ('afm',)}[fontext]
 
+
 def list_fonts(directory, extensions):
     """
     Return a list of all fonts matching any of the extensions,
@@ -169,6 +178,7 @@ def list_fonts(directory, extensions):
                         for ext in extensions])
     return cbook.listFiles(directory, pattern)
 
+
 def win32FontDirectory():
     """
     Return the user-specified font directory for Win32.  This is
@@ -181,7 +191,7 @@ def win32FontDirectory():
     try:
         from six.moves import winreg
     except ImportError:
-        pass # Fall through to default
+        pass  # Fall through to default
     else:
         try:
             user = winreg.OpenKey(winreg.HKEY_CURRENT_USER, MSFolders)
@@ -189,13 +199,14 @@ def win32FontDirectory():
                 try:
                     return winreg.QueryValueEx(user, 'Fonts')[0]
                 except OSError:
-                    pass # Fall through to default
+                    pass  # Fall through to default
             finally:
                 winreg.CloseKey(user)
         except OSError:
-            pass # Fall through to default
+            pass  # Fall through to default
     return os.path.join(os.environ['WINDIR'], 'Fonts')
 
+
 def win32InstalledFonts(directory=None, fontext='ttf'):
     """
     Search for fonts in the specified font directory, or use the
@@ -241,6 +252,7 @@ def win32InstalledFonts(directory=None, fontext='ttf'):
             winreg.CloseKey(local)
     return None
 
+
 def OSXInstalledFonts(directories=None, fontext='ttf'):
     """
     Get list of font files on OS X - ignores font suffix by default.
@@ -258,38 +270,40 @@ def OSXInstalledFonts(directories=None, fontext='ttf'):
             files.extend(list_fonts(path, fontext))
     return files
 
-def get_fontconfig_fonts(fontext='ttf'):
+
+@lru_cache()
+def _call_fc_list():
+    """Cache and list the font filenames known to `fc-list`.
     """
-    Grab a list of all the fonts that are being tracked by fontconfig
-    by making a system call to ``fc-list``.  This is an easy way to
-    grab all of the fonts the user wants to be made available to
-    applications, without needing knowing where all of them reside.
+    # Delay the warning by 5s.
+    timer = Timer(5, lambda: warnings.warn(
+        'Matplotlib is building the font cache using fc-list. '
+        'This may take a moment.'))
+    timer.start()
+    try:
+        out = subprocess.check_output([str('fc-list'), '--format=%{file}\\n'])
+    except (OSError, subprocess.CalledProcessError):
+        return []
+    finally:
+        timer.cancel()
+    fnames = []
+    for fname in out.split(b'\n'):
+        try:
+            fname = six.text_type(fname, sys.getfilesystemencoding())
+        except UnicodeDecodeError:
+            continue
+        fnames.append(fname)
+    return fnames
+
+
+def get_fontconfig_fonts(fontext='ttf'):
+    """List the font filenames known to `fc-list` having the given extension.
     """
     fontext = get_fontext_synonyms(fontext)
+    return [fname for fname in _call_fc_list()
+            if os.path.splitext(fname)[1][1:] in fontext]
 
-    fontfiles = {}
-    try:
-        pipe = subprocess.Popen(['fc-list', '--format=%{file}\\n'],
-                                stdout=subprocess.PIPE,
-                                stderr=subprocess.PIPE)
-        output = pipe.communicate()[0]
-    except (OSError, IOError):
-        # Calling fc-list did not work, so we'll just return nothing
-        return fontfiles
 
-    if pipe.returncode == 0:
-        # The line breaks between results are in ascii, but each entry
-        # is in in sys.filesystemencoding().
-        for fname in output.split(b'\n'):
-            try:
-                fname = six.text_type(fname, sys.getfilesystemencoding())
-            except UnicodeDecodeError:
-                continue
-            if (os.path.splitext(fname)[1][1:] in fontext and
-                os.path.exists(fname)):
-                fontfiles[fname] = 1
-
-    return fontfiles
 
 def findSystemFonts(fontpaths=None, fontext='ttf'):
     """
@@ -299,7 +313,7 @@ def findSystemFonts(fontpaths=None, fontext='ttf'):
     available.  A list of TrueType fonts are returned by default with
     AFM fonts as an option.
     """
-    fontfiles = {}
+    fontfiles = set()
     fontexts = get_fontext_synonyms(fontext)
 
     if fontpaths is None:
@@ -311,16 +325,16 @@ def findSystemFonts(fontpaths=None, fontext='ttf'):
             for f in win32InstalledFonts(fontdir):
                 base, ext = os.path.splitext(f)
                 if len(ext)>1 and ext[1:].lower() in fontexts:
-                    fontfiles[f] = 1
+                    fontfiles.add(f)
         else:
             fontpaths = X11FontDirectories
             # check for OS X & load its fonts if present
             if sys.platform == 'darwin':
                 for f in OSXInstalledFonts(fontext=fontext):
-                    fontfiles[f] = 1
+                    fontfiles.add(f)
 
             for f in get_fontconfig_fonts(fontext):
-                fontfiles[f] = 1
+                fontfiles.add(f)
 
     elif isinstance(fontpaths, six.string_types):
         fontpaths = [fontpaths]
@@ -328,9 +342,10 @@ def findSystemFonts(fontpaths=None, fontext='ttf'):
     for path in fontpaths:
         files = list_fonts(path, fontexts)
         for fname in files:
-            fontfiles[os.path.abspath(fname)] = 1
+            fontfiles.add(os.path.abspath(fname))
+
+    return [fname for fname in fontfiles if os.path.exists(fname)]
 
-    return [fname for fname in six.iterkeys(fontfiles) if os.path.exists(fname)]
 
 def weight_as_number(weight):
     """
@@ -413,7 +428,6 @@ def ttfFontProperty(font):
     else:
         style = 'normal'
 
-
     #  Variants are: small-caps and normal (default)
 
     #  !!!!  Untested
@@ -445,8 +459,8 @@ def ttfFontProperty(font):
     #  Relative stretches are: wider, narrower
     #  Child value is: inherit
 
-    if sfnt4.find('narrow') >= 0 or sfnt4.find('condensed') >= 0 or \
-           sfnt4.find('cond') >= 0:
+    if (sfnt4.find('narrow') >= 0 or sfnt4.find('condensed') >= 0 or
+            sfnt4.find('cond') >= 0):
         stretch = 'condensed'
     elif sfnt4.find('demi cond') >= 0:
         stretch = 'semi-condensed'
@@ -554,8 +568,10 @@ def createFontList(fontfiles, fontext='ttf'):
     for fpath in fontfiles:
         verbose.report('createFontDict: %s' % (fpath), 'debug')
         fname = os.path.split(fpath)[1]
-        if fname in seen:  continue
-        else: seen[fname] = 1
+        if fname in seen:
+            continue
+        else:
+            seen[fname] = 1
         if fontext == 'afm':
             try:
                 fh = open(fpath, 'rb')
@@ -568,7 +584,7 @@ def createFontList(fontfiles, fontext='ttf'):
                 finally:
                     fh.close()
             except RuntimeError:
-                verbose.report("Could not parse font file %s"%fpath)
+                verbose.report("Could not parse font file %s" % fpath)
                 continue
             try:
                 prop = afmFontProperty(fpath, font)
@@ -578,20 +594,24 @@ def createFontList(fontfiles, fontext='ttf'):
             try:
                 font = ft2font.FT2Font(fpath)
             except RuntimeError:
-                verbose.report("Could not open font file %s"%fpath)
+                verbose.report("Could not open font file %s" % fpath)
                 continue
             except UnicodeError:
                 verbose.report("Cannot handle unicode filenames")
-                #print >> sys.stderr, 'Bad file is', fpath
+                # print >> sys.stderr, 'Bad file is', fpath
+                continue
+            except IOError:
+                verbose.report("IO error - cannot open font file %s" % fpath)
                 continue
             try:
                 prop = ttfFontProperty(font)
-            except KeyError:
+            except (KeyError, RuntimeError, ValueError):
                 continue
 
         fontlist.append(prop)
     return fontlist
 
+
 class FontProperties(object):
     """
     A class for storing and manipulating font properties.
@@ -639,10 +659,10 @@ class FontProperties(object):
     on the font manager's default font size.
 
     This class will also accept a `fontconfig
-    <http://www.fontconfig.org/>`_ pattern, if it is the only argument
-    provided.  See the documentation on `fontconfig patterns
-    <http://www.fontconfig.org/fontconfig-user.html>`_.  This support
-    does not require fontconfig to be installed.  We are merely
+    <https://www.freedesktop.org/wiki/Software/fontconfig/>`_ pattern, if it is
+    the only argument provided.  See the documentation on `fontconfig patterns
+    <https://www.freedesktop.org/software/fontconfig/fontconfig-user.html>`_.
+    This support does not require fontconfig to be installed.  We are merely
     borrowing its pattern syntax for use here.
 
     Note that matplotlib's internal font manager and fontconfig use a
@@ -661,12 +681,12 @@ class FontProperties(object):
                  fname  = None, # if this is set, it's a hardcoded filename to use
                  _init   = None  # used only by copy()
                  ):
-        self._family = None
-        self._slant = None
-        self._variant = None
-        self._weight = None
-        self._stretch = None
-        self._size = None
+        self._family = _normalize_font_family(rcParams['font.family'])
+        self._slant = rcParams['font.style']
+        self._variant = rcParams['font.variant']
+        self._weight = rcParams['font.weight']
+        self._stretch = rcParams['font.stretch']
+        self._size = rcParams['font.size']
         self._file = None
 
         # This is used only by copy()
@@ -720,11 +740,6 @@ class FontProperties(object):
         """
         Return a list of font names that comprise the font family.
         """
-        if self._family is None:
-            family = rcParams['font.family']
-            if is_string_like(family):
-                return [family]
-            return family
         return self._family
 
     def get_name(self):
@@ -732,15 +747,13 @@ class FontProperties(object):
         Return the name of the font that best matches the font
         properties.
         """
-        return ft2font.FT2Font(findfont(self)).family_name
+        return get_font(findfont(self)).family_name
 
     def get_style(self):
         """
         Return the font style.  Values are: 'normal', 'italic' or
         'oblique'.
         """
-        if self._slant is None:
-            return rcParams['font.style']
         return self._slant
     get_slant = get_style
 
@@ -749,8 +762,6 @@ class FontProperties(object):
         Return the font variant.  Values are: 'normal' or
         'small-caps'.
         """
-        if self._variant is None:
-            return rcParams['font.variant']
         return self._variant
 
     def get_weight(self):
@@ -760,8 +771,6 @@ class FontProperties(object):
         'medium', 'roman', 'semibold', 'demibold', 'demi', 'bold',
         'heavy', 'extra bold', 'black'
         """
-        if self._weight is None:
-            return rcParams['font.weight']
         return self._weight
 
     def get_stretch(self):
@@ -770,26 +779,16 @@ class FontProperties(object):
         'extra-condensed', 'condensed', 'semi-condensed', 'normal',
         'semi-expanded', 'expanded', 'extra-expanded', 'ultra-expanded'.
         """
-        if self._stretch is None:
-            return rcParams['font.stretch']
         return self._stretch
 
     def get_size(self):
         """
         Return the font size.
         """
-        if self._size is None:
-            return rcParams['font.size']
         return self._size
 
     def get_size_in_points(self):
-        if self._size is not None:
-            try:
-                return float(self._size)
-            except ValueError:
-                pass
-        default_size = FontManager.get_default_size()
-        return default_size * font_scalings.get(self._size)
+        return self._size
 
     def get_file(self):
         """
@@ -803,7 +802,7 @@ class FontProperties(object):
         specified with fontconfig's ``fc-match`` utility.
 
         See the documentation on `fontconfig patterns
-        <http://www.fontconfig.org/fontconfig-user.html>`_.
+        <https://www.freedesktop.org/software/fontconfig/fontconfig-user.html>`_.
 
         This support does not require fontconfig to be installed or
         support for it to be enabled.  We are merely borrowing its
@@ -821,9 +820,7 @@ class FontProperties(object):
         """
         if family is None:
             family = rcParams['font.family']
-        if is_string_like(family):
-            family = [family]
-        self._family = family
+        self._family = _normalize_font_family(family)
     set_name = set_family
 
     def set_style(self, style):
@@ -833,7 +830,7 @@ class FontProperties(object):
         """
         if style is None:
             style = rcParams['font.style']
-        if style not in ('normal', 'italic', 'oblique', None):
+        if style not in ('normal', 'italic', 'oblique'):
             raise ValueError("style must be normal, italic or oblique")
         self._slant = style
     set_slant = set_style
@@ -844,7 +841,7 @@ class FontProperties(object):
         """
         if variant is None:
             variant = rcParams['font.variant']
-        if variant not in ('normal', 'small-caps', None):
+        if variant not in ('normal', 'small-caps'):
             raise ValueError("variant must be normal or small-caps")
         self._variant = variant
 
@@ -864,6 +861,7 @@ class FontProperties(object):
         except ValueError:
             if weight not in weight_dict:
                 raise ValueError("weight is invalid")
+            weight = weight_dict[weight]
         self._weight = weight
 
     def set_stretch(self, stretch):
@@ -895,8 +893,15 @@ class FontProperties(object):
         try:
             size = float(size)
         except ValueError:
-            if size is not None and size not in font_scalings:
-                raise ValueError("size is invalid")
+            try:
+                scale = font_scalings[size]
+            except KeyError:
+                raise ValueError(
+                    "Size is invalid. Valid font size are " + ", ".join(
+                        str(i) for i in font_scalings.keys()))
+            else:
+                size = scale * FontManager.get_default_size()
+
         self._size = size
 
     def set_file(self, file):
@@ -911,7 +916,7 @@ class FontProperties(object):
         Set the properties by parsing a fontconfig *pattern*.
 
         See the documentation on `fontconfig patterns
-        <http://www.fontconfig.org/fontconfig-user.html>`_.
+        <https://www.freedesktop.org/software/fontconfig/fontconfig-user.html>`_.
 
         This support does not require fontconfig to be installed or
         support for it to be enabled.  We are merely borrowing its
@@ -925,7 +930,8 @@ class FontProperties(object):
 
     def copy(self):
         """Return a deep copy of self"""
-        return FontProperties(_init = self)
+        return FontProperties(_init=self)
+
 
 def ttfdict_to_fnames(d):
     """
@@ -941,6 +947,7 @@ def ttfdict_to_fnames(d):
                             fnames.append(fname)
     return fnames
 
+
 def pickle_dump(data, filename):
     """
     Equivalent to pickle.dump(data, open(filename, 'w'))
@@ -949,6 +956,7 @@ def pickle_dump(data, filename):
     with open(filename, 'wb') as fh:
         pickle.dump(data, fh)
 
+
 def pickle_load(filename):
     """
     Equivalent to pickle.load(open(filename, 'r'))
@@ -959,6 +967,14 @@ def pickle_load(filename):
     return data
 
 
+def _normalize_font_family(family):
+    if is_string_like(family):
+        family = [six.text_type(family)]
+    elif isinstance(family, Iterable):
+        family = [six.text_type(f) for f in family]
+    return family
+
+
 class TempCache(object):
     """
     A class to store temporary caches that are (a) not saved to disk
@@ -1008,7 +1024,7 @@ class FontManager(object):
     # Increment this version number whenever the font cache data
     # format or behavior has changed and requires a existing font
     # cache files to be rebuilt.
-    __version__ = 101
+    __version__ = 201
 
     def __init__(self, size=None, weight='normal'):
         self._version = self.__version__
@@ -1036,13 +1052,13 @@ class FontManager(object):
 
         self.ttffiles = findSystemFonts(paths) + findSystemFonts()
         self.defaultFamily = {
-            'ttf': 'Bitstream Vera Sans',
+            'ttf': 'DejaVu Sans',
             'afm': 'Helvetica'}
         self.defaultFont = {}
 
         for fname in self.ttffiles:
             verbose.report('trying fontname %s' % fname, 'debug')
-            if fname.lower().find('vera.ttf')>=0:
+            if fname.lower().find('DejaVuSans.ttf')>=0:
                 self.defaultFont['ttf'] = fname
                 break
         else:
@@ -1093,15 +1109,18 @@ class FontManager(object):
         Returns a match score between the list of font families in
         *families* and the font family name *family2*.
 
-        An exact match anywhere in the list returns 0.0.
+        An exact match at the head of the list returns 0.0.
 
-        A match by generic font name will return 0.1.
+        A match further down the list will return between 0 and 1.
 
         No match will return 1.0.
         """
         if not isinstance(families, (list, tuple)):
             families = [families]
+        elif len(families) == 0:
+            return 1.0
         family2 = family2.lower()
+        step = 1 / len(families)
         for i, family1 in enumerate(families):
             family1 = family1.lower()
             if family1 in font_family_aliases:
@@ -1111,12 +1130,11 @@ class FontManager(object):
                 options = [x.lower() for x in options]
                 if family2 in options:
                     idx = options.index(family2)
-                    return ((0.1 * (idx / len(options))) *
-                            ((i + 1) / len(families)))
+                    return (i + (idx / len(options))) * step
             elif family1 == family2:
                 # The score should be weighted by where in the
                 # list the font was found.
-                return i / len(families)
+                return i * step
         return 1.0
 
     def score_style(self, style1, style2):
@@ -1217,7 +1235,7 @@ class FontManager(object):
         font is given a similarity score to the target font
         properties.  The first font with the highest score is
         returned.  If no matches below a certain threshold are found,
-        the default font (usually Vera Sans) is returned.
+        the default font (usually DejaVu Sans) is returned.
 
         `directory`, is specified, will only return fonts from the
         given directory (or subdirectory of that directory).
@@ -1226,7 +1244,7 @@ class FontManager(object):
         perform the O(n) nearest neighbor search.
 
         If `fallback_to_default` is True, will fallback to the default
-        font family (usually "Bitstream Vera Sans" or "Helvetica") if
+        font family (usually "DejaVu Sans" or "Helvetica") if
         the first lookup hard-fails.
 
         See the `W3C Cascading Style Sheet, Level 1
@@ -1282,7 +1300,7 @@ class FontManager(object):
                 return self.findfont(default_prop, fontext, directory, False)
             else:
                 # This is a hard fail -- we can't find anything reasonable,
-                # so just return the vera.ttf
+                # so just return the DejuVuSans.ttf
                 warnings.warn(
                     'findfont: Could not match %s. Returning %s' %
                     (prop, self.defaultFont[fontext]),
@@ -1308,7 +1326,6 @@ class FontManager(object):
             _lookup_cache[fontext].set(prop, result)
         return result
 
-
 _is_opentype_cff_font_cache = {}
 def is_opentype_cff_font(filename):
     """
@@ -1321,7 +1338,7 @@ def is_opentype_cff_font(filename):
         if result is None:
             with open(filename, 'rb') as fd:
                 tag = fd.read(4)
-            result = (tag == 'OTTO')
+            result = (tag == b'OTTO')
             _is_opentype_cff_font_cache[filename] = result
         return result
     return False
@@ -1329,6 +1346,10 @@ def is_opentype_cff_font(filename):
 fontManager = None
 _fmcache = None
 
+
+get_font = lru_cache(64)(ft2font.FT2Font)
+
+
 # The experimental fontconfig-based backend.
 if USE_FONTCONFIG and sys.platform != 'win32':
     import re
@@ -1377,13 +1398,12 @@ if USE_FONTCONFIG and sys.platform != 'win32':
 else:
     _fmcache = None
 
-    if not 'TRAVIS' in os.environ:
-        cachedir = get_cachedir()
-        if cachedir is not None:
-            if six.PY3:
-                _fmcache = os.path.join(cachedir, 'fontList.py3k.cache')
-            else:
-                _fmcache = os.path.join(cachedir, 'fontList.cache')
+    cachedir = get_cachedir()
+    if cachedir is not None:
+        if six.PY3:
+            _fmcache = os.path.join(cachedir, 'fontList.py3k.cache')
+        else:
+            _fmcache = os.path.join(cachedir, 'fontList.cache')
 
     fontManager = None
 
@@ -1394,9 +1414,14 @@ else:
 
     def _rebuild():
         global fontManager
+
         fontManager = FontManager()
+
         if _fmcache:
-            pickle_dump(fontManager, _fmcache)
+            with cbook.Locked(cachedir):
+                pickle_dump(fontManager, _fmcache)
+
+
         verbose.report("generated new fontManager")
 
     if _fmcache:
@@ -1408,6 +1433,8 @@ else:
             else:
                 fontManager.default_size = None
                 verbose.report("Using fontManager instance from %s" % _fmcache)
+        except cbook.Locked.TimeoutError:
+            raise
         except:
             _rebuild()
     else:
