@@ -35,6 +35,7 @@ import numpy as np
 from matplotlib.lines import Line2D
 from matplotlib.patches import Rectangle
 import matplotlib.collections as mcoll
+import matplotlib.colors as mcolors
 
 
 def update_from_first_child(tgt, src):
@@ -264,13 +265,12 @@ class HandlerLineCollection(HandlerLine2D):
             return self._numpoints
 
     def _default_update_prop(self, legend_handle, orig_handle):
-        lw = orig_handle.get_linewidth()[0]
-        dashes = orig_handle.get_dashes()[0]
+        lw = orig_handle.get_linewidths()[0]
+        dashes = orig_handle._us_linestyles[0]
         color = orig_handle.get_colors()[0]
         legend_handle.set_color(color)
+        legend_handle.set_linestyle(dashes)
         legend_handle.set_linewidth(lw)
-        if dashes[0] is not None: # dashed line
-            legend_handle.set_dashes(dashes[1])
 
     def create_artists(self, legend, orig_handle,
                        xdescent, ydescent, width, height, fontsize, trans):
@@ -304,13 +304,16 @@ class HandlerRegularPolyCollection(HandlerNpointsYoffsets):
     def get_sizes(self, legend, orig_handle,
                  xdescent, ydescent, width, height, fontsize):
         if self._sizes is None:
-            size_max = max(orig_handle.get_sizes()) * legend.markerscale ** 2
-            size_min = min(orig_handle.get_sizes()) * legend.markerscale ** 2
+            handle_sizes = orig_handle.get_sizes()
+            if not len(handle_sizes):
+                handle_sizes = [1]
+            size_max = max(handle_sizes) * legend.markerscale ** 2
+            size_min = min(handle_sizes) * legend.markerscale ** 2
 
             numpoints = self.get_numpoints(legend)
             if numpoints < 4:
                 sizes = [.5 * (size_max + size_min), size_max,
-                         size_min]
+                         size_min][:numpoints]
             else:
                 rng = (size_max - size_min)
                 sizes = rng * np.linspace(0, 1, numpoints) + size_min
@@ -581,3 +584,44 @@ class HandlerTuple(HandlerBase):
             a_list.extend(_a_list)
 
         return a_list
+
+
+class HandlerPolyCollection(HandlerBase):
+    """
+    Handler for PolyCollection used in fill_between and stackplot.
+    """
+    def _update_prop(self, legend_handle, orig_handle):
+        def first_color(colors):
+            if colors is None:
+                return None
+            colors = mcolors.to_rgba_array(colors)
+            if len(colors):
+                return colors[0]
+            else:
+                return "none"
+        def get_first(prop_array):
+            if len(prop_array):
+                return prop_array[0]
+            else:
+                return None
+        edgecolor = getattr(orig_handle, '_original_edgecolor',
+                            orig_handle.get_edgecolor())
+        legend_handle.set_edgecolor(first_color(edgecolor))
+        facecolor = getattr(orig_handle, '_original_facecolor',
+                            orig_handle.get_facecolor())
+        legend_handle.set_facecolor(first_color(facecolor))
+        legend_handle.set_fill(orig_handle.get_fill())
+        legend_handle.set_hatch(orig_handle.get_hatch())
+        legend_handle.set_linewidth(get_first(orig_handle.get_linewidths()))
+        legend_handle.set_linestyle(get_first(orig_handle.get_linestyles()))
+        legend_handle.set_transform(get_first(orig_handle.get_transforms()))
+        legend_handle.set_figure(orig_handle.get_figure())
+        legend_handle.set_alpha(orig_handle.get_alpha())
+
+    def create_artists(self, legend, orig_handle,
+                       xdescent, ydescent, width, height, fontsize, trans):
+        p = Rectangle(xy=(-xdescent, -ydescent),
+                      width=width, height=height)
+        self.update_prop(p, orig_handle, legend)
+        p.set_transform(trans)
+        return [p]
