@@ -1,7 +1,7 @@
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-from matplotlib.externals import six
+import six
 import itertools
 from distutils.version import LooseVersion as V
 
@@ -17,8 +17,11 @@ import numpy as np
 from numpy.testing.utils import assert_array_equal, assert_array_almost_equal
 from nose.plugins.skip import SkipTest
 
+from matplotlib import cycler
+import matplotlib
 import matplotlib.colors as mcolors
 import matplotlib.cm as cm
+import matplotlib.colorbar as mcolorbar
 import matplotlib.cbook as cbook
 import matplotlib.pyplot as plt
 from matplotlib.testing.decorators import (image_comparison,
@@ -191,6 +194,24 @@ def test_Normalize():
     _scalar_tester(norm, vals)
     _mask_tester(norm, vals)
 
+    # Handle integer input correctly (don't overflow when computing max-min,
+    # i.e. 127-(-128) here).
+    vals = np.array([-128, 127], dtype=np.int8)
+    norm = mcolors.Normalize(vals.min(), vals.max())
+    assert_array_equal(np.asarray(norm(vals)), [0, 1])
+
+    # Don't lose precision on longdoubles (float128 on Linux):
+    # for array inputs...
+    vals = np.array([1.2345678901, 9.8765432109], dtype=np.longdouble)
+    norm = mcolors.Normalize(vals.min(), vals.max())
+    assert_array_equal(np.asarray(norm(vals)), [0, 1])
+    # and for scalar ones.
+    eps = np.finfo(np.longdouble).resolution
+    norm = plt.Normalize(1, 1 + 100 * eps)
+    # This returns exactly 0.5 when longdouble is extended precision (80-bit),
+    # but only a value close to it when it is quadruple precision (128-bit).
+    assert 0 < norm(1 + 50 * eps) < 1
+
 
 def test_SymLogNorm():
     """
@@ -211,6 +232,17 @@ def test_SymLogNorm():
     assert_array_almost_equal(normed_vals, expected)
 
 
+@cleanup
+def test_SymLogNorm_colorbar():
+    """
+    Test un-called SymLogNorm in a colorbar.
+    """
+    norm = mcolors.SymLogNorm(0.1, vmin=-1, vmax=1, linscale=1)
+    fig = plt.figure()
+    cbar = mcolorbar.ColorbarBase(fig.add_subplot(111), norm=norm)
+    plt.close(fig)
+
+
 def _inverse_tester(norm_instance, vals):
     """
     Checks if the inverse of the given normalization is working.
@@ -257,7 +289,7 @@ def test_cmap_and_norm_from_levels_and_colors():
 def test_cmap_and_norm_from_levels_and_colors2():
     levels = [-1, 2, 2.5, 3]
     colors = ['red', (0, 1, 0), 'blue', (0.5, 0.5, 0.5), (0.0, 0.0, 0.0, 1.0)]
-    clr = mcolors.colorConverter.to_rgba_array(colors)
+    clr = mcolors.to_rgba_array(colors)
     bad = (0.1, 0.1, 0.1, 0.1)
     no_color = (0.0, 0.0, 0.0, 0.0)
     masked_value = 'masked_value'
@@ -334,13 +366,9 @@ def test_autoscale_masked():
 def test_colors_no_float():
     # Gray must be a string to distinguish 3-4 grays from RGB or RGBA.
 
-    def gray_from_float_rgb():
-        return mcolors.colorConverter.to_rgb(0.4)
-
     def gray_from_float_rgba():
-        return mcolors.colorConverter.to_rgba(0.4)
+        return mcolors.to_rgba(0.4)
 
-    assert_raises(ValueError, gray_from_float_rgb)
     assert_raises(ValueError, gray_from_float_rgba)
 
 
@@ -381,41 +409,44 @@ def test_light_source_shading_default():
     rgb = ls.shade(z, cmap)
 
     # Result stored transposed and rounded for for more compact display...
-    expect = np.array([[[0.87, 0.85, 0.90, 0.90, 0.82, 0.62, 0.34, 0.00],
-                        [0.85, 0.94, 0.99, 1.00, 1.00, 0.96, 0.62, 0.17],
-                        [0.90, 0.99, 1.00, 1.00, 1.00, 1.00, 0.71, 0.33],
-                        [0.90, 1.00, 1.00, 1.00, 1.00, 0.98, 0.51, 0.29],
-                        [0.82, 1.00, 1.00, 1.00, 1.00, 0.64, 0.25, 0.13],
-                        [0.62, 0.96, 1.00, 0.98, 0.64, 0.22, 0.06, 0.03],
-                        [0.34, 0.62, 0.71, 0.51, 0.25, 0.06, 0.00, 0.01],
-                        [0.00, 0.17, 0.33, 0.29, 0.13, 0.03, 0.01, 0.00]],
+    expect = np.array(
+        [[[0.00, 0.45, 0.90, 0.90, 0.82, 0.62, 0.28, 0.00],
+          [0.45, 0.94, 0.99, 1.00, 1.00, 0.96, 0.65, 0.17],
+          [0.90, 0.99, 1.00, 1.00, 1.00, 1.00, 0.94, 0.35],
+          [0.90, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 0.49],
+          [0.82, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 0.41],
+          [0.62, 0.96, 1.00, 1.00, 1.00, 1.00, 0.90, 0.07],
+          [0.28, 0.65, 0.94, 1.00, 1.00, 0.90, 0.35, 0.01],
+          [0.00, 0.17, 0.35, 0.49, 0.41, 0.07, 0.01, 0.00]],
 
-                       [[0.87, 0.79, 0.83, 0.80, 0.66, 0.44, 0.23, 0.00],
-                        [0.79, 0.88, 0.93, 0.92, 0.83, 0.66, 0.38, 0.10],
-                        [0.83, 0.93, 0.99, 1.00, 0.92, 0.75, 0.40, 0.18],
-                        [0.80, 0.92, 1.00, 0.99, 0.93, 0.75, 0.28, 0.14],
-                        [0.66, 0.83, 0.92, 0.93, 0.87, 0.44, 0.12, 0.06],
-                        [0.44, 0.66, 0.75, 0.75, 0.44, 0.12, 0.03, 0.01],
-                        [0.23, 0.38, 0.40, 0.28, 0.12, 0.03, 0.00, 0.00],
-                        [0.00, 0.10, 0.18, 0.14, 0.06, 0.01, 0.00, 0.00]],
+         [[0.00, 0.28, 0.59, 0.72, 0.62, 0.40, 0.18, 0.00],
+          [0.28, 0.78, 0.93, 0.92, 0.83, 0.66, 0.39, 0.11],
+          [0.59, 0.93, 0.99, 1.00, 0.92, 0.75, 0.50, 0.21],
+          [0.72, 0.92, 1.00, 0.99, 0.93, 0.76, 0.51, 0.18],
+          [0.62, 0.83, 0.92, 0.93, 0.87, 0.68, 0.42, 0.08],
+          [0.40, 0.66, 0.75, 0.76, 0.68, 0.52, 0.23, 0.02],
+          [0.18, 0.39, 0.50, 0.51, 0.42, 0.23, 0.00, 0.00],
+          [0.00, 0.11, 0.21, 0.18, 0.08, 0.02, 0.00, 0.00]],
 
-                       [[0.87, 0.75, 0.78, 0.73, 0.55, 0.33, 0.16, 0.00],
-                        [0.75, 0.85, 0.90, 0.86, 0.71, 0.48, 0.23, 0.05],
-                        [0.78, 0.90, 0.98, 1.00, 0.82, 0.51, 0.21, 0.08],
-                        [0.73, 0.86, 1.00, 0.97, 0.84, 0.47, 0.11, 0.05],
-                        [0.55, 0.71, 0.82, 0.84, 0.71, 0.20, 0.03, 0.01],
-                        [0.33, 0.48, 0.51, 0.47, 0.20, 0.02, 0.00, 0.00],
-                        [0.16, 0.23, 0.21, 0.11, 0.03, 0.00, 0.00, 0.00],
-                        [0.00, 0.05, 0.08, 0.05, 0.01, 0.00, 0.00, 0.00]],
+         [[0.00, 0.18, 0.38, 0.46, 0.39, 0.26, 0.11, 0.00],
+          [0.18, 0.50, 0.70, 0.75, 0.64, 0.44, 0.25, 0.07],
+          [0.38, 0.70, 0.91, 0.98, 0.81, 0.51, 0.29, 0.13],
+          [0.46, 0.75, 0.98, 0.96, 0.84, 0.48, 0.22, 0.12],
+          [0.39, 0.64, 0.81, 0.84, 0.71, 0.31, 0.11, 0.05],
+          [0.26, 0.44, 0.51, 0.48, 0.31, 0.10, 0.03, 0.01],
+          [0.11, 0.25, 0.29, 0.22, 0.11, 0.03, 0.00, 0.00],
+          [0.00, 0.07, 0.13, 0.12, 0.05, 0.01, 0.00, 0.00]],
+
+         [[1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00]]
+        ]).T
 
-                       [[1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00]]]).T
     if (V(np.__version__) == V('1.9.0')):
         # Numpy 1.9.0 uses a 2. order algorithm on the edges by default
         # This was changed back again in 1.9.1
@@ -443,41 +474,43 @@ def test_light_source_masked_shading():
     rgb = ls.shade(z, cmap)
 
     # Result stored transposed and rounded for for more compact display...
-    expect = np.array([[[0.90, 0.88, 0.91, 0.91, 0.84, 0.64, 0.36, 0.00],
-                        [0.88, 0.96, 1.00, 1.00, 1.00, 0.97, 0.64, 0.18],
-                        [0.91, 1.00, 1.00, 1.00, 1.00, 1.00, 0.74, 0.34],
-                        [0.91, 1.00, 1.00, 0.00, 0.00, 1.00, 0.52, 0.30],
-                        [0.84, 1.00, 1.00, 0.00, 0.00, 1.00, 0.25, 0.13],
-                        [0.64, 0.97, 1.00, 1.00, 1.00, 0.23, 0.07, 0.03],
-                        [0.36, 0.64, 0.74, 0.52, 0.25, 0.07, 0.00, 0.01],
-                        [0.00, 0.18, 0.34, 0.30, 0.13, 0.03, 0.01, 0.00]],
+    expect = np.array(
+        [[[0.00, 0.46, 0.91, 0.91, 0.84, 0.64, 0.29, 0.00],
+          [0.46, 0.96, 1.00, 1.00, 1.00, 0.97, 0.67, 0.18],
+          [0.91, 1.00, 1.00, 1.00, 1.00, 1.00, 0.96, 0.36],
+          [0.91, 1.00, 1.00, 0.00, 0.00, 1.00, 1.00, 0.51],
+          [0.84, 1.00, 1.00, 0.00, 0.00, 1.00, 1.00, 0.44],
+          [0.64, 0.97, 1.00, 1.00, 1.00, 1.00, 0.94, 0.09],
+          [0.29, 0.67, 0.96, 1.00, 1.00, 0.94, 0.38, 0.01],
+          [0.00, 0.18, 0.36, 0.51, 0.44, 0.09, 0.01, 0.00]],
 
-                       [[0.90, 0.82, 0.85, 0.82, 0.68, 0.46, 0.24, 0.00],
-                        [0.82, 0.91, 0.95, 0.93, 0.85, 0.68, 0.39, 0.10],
-                        [0.85, 0.95, 1.00, 0.78, 0.78, 0.77, 0.42, 0.18],
-                        [0.82, 0.93, 0.78, 0.00, 0.00, 0.78, 0.30, 0.15],
-                        [0.68, 0.85, 0.78, 0.00, 0.00, 0.78, 0.13, 0.06],
-                        [0.46, 0.68, 0.77, 0.78, 0.78, 0.13, 0.03, 0.01],
-                        [0.24, 0.39, 0.42, 0.30, 0.13, 0.03, 0.00, 0.00],
-                        [0.00, 0.10, 0.18, 0.15, 0.06, 0.01, 0.00, 0.00]],
+         [[0.00, 0.29, 0.61, 0.75, 0.64, 0.41, 0.18, 0.00],
+          [0.29, 0.81, 0.95, 0.93, 0.85, 0.68, 0.40, 0.11],
+          [0.61, 0.95, 1.00, 0.78, 0.78, 0.77, 0.52, 0.22],
+          [0.75, 0.93, 0.78, 0.00, 0.00, 0.78, 0.54, 0.19],
+          [0.64, 0.85, 0.78, 0.00, 0.00, 0.78, 0.45, 0.08],
+          [0.41, 0.68, 0.77, 0.78, 0.78, 0.55, 0.25, 0.02],
+          [0.18, 0.40, 0.52, 0.54, 0.45, 0.25, 0.00, 0.00],
+          [0.00, 0.11, 0.22, 0.19, 0.08, 0.02, 0.00, 0.00]],
 
-                       [[0.90, 0.79, 0.81, 0.76, 0.58, 0.35, 0.17, 0.00],
-                        [0.79, 0.88, 0.92, 0.88, 0.73, 0.50, 0.24, 0.05],
-                        [0.81, 0.92, 1.00, 0.50, 0.50, 0.53, 0.22, 0.09],
-                        [0.76, 0.88, 0.50, 0.00, 0.00, 0.50, 0.12, 0.05],
-                        [0.58, 0.73, 0.50, 0.00, 0.00, 0.50, 0.03, 0.01],
-                        [0.35, 0.50, 0.53, 0.50, 0.50, 0.02, 0.00, 0.00],
-                        [0.17, 0.24, 0.22, 0.12, 0.03, 0.00, 0.00, 0.00],
-                        [0.00, 0.05, 0.09, 0.05, 0.01, 0.00, 0.00, 0.00]],
+         [[0.00, 0.19, 0.39, 0.48, 0.41, 0.26, 0.12, 0.00],
+          [0.19, 0.52, 0.73, 0.78, 0.66, 0.46, 0.26, 0.07],
+          [0.39, 0.73, 0.95, 0.50, 0.50, 0.53, 0.30, 0.14],
+          [0.48, 0.78, 0.50, 0.00, 0.00, 0.50, 0.23, 0.12],
+          [0.41, 0.66, 0.50, 0.00, 0.00, 0.50, 0.11, 0.05],
+          [0.26, 0.46, 0.53, 0.50, 0.50, 0.11, 0.03, 0.01],
+          [0.12, 0.26, 0.30, 0.23, 0.11, 0.03, 0.00, 0.00],
+          [0.00, 0.07, 0.14, 0.12, 0.05, 0.01, 0.00, 0.00]],
 
-                       [[1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 0.00, 0.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 0.00, 0.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
-                        [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00]]]).T
+         [[1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 0.00, 0.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 0.00, 0.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00],
+          [1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00, 1.00]],
+        ]).T
 
     assert_array_almost_equal(rgb, expect, decimal=2)
 
@@ -550,6 +583,12 @@ def test_light_source_planar_hillshading():
             assert_array_almost_equal(h, np.cos(np.radians(angle)))
 
 
+def test_color_names():
+    assert mcolors.to_hex("blue") == "#0000ff"
+    assert mcolors.to_hex("xkcd:blue") == "#0343df"
+    assert mcolors.to_hex("tab:blue") == "#1f77b4"
+
+
 def _sph2cart(theta, phi):
     x = np.cos(theta) * np.sin(phi)
     y = np.sin(theta) * np.sin(phi)
@@ -582,6 +621,62 @@ def test_pandas_iterable():
     assert_sequence_equal(cm1.colors, cm2.colors)
 
 
+@cleanup
+def test_cn():
+    matplotlib.rcParams['axes.prop_cycle'] = cycler('color',
+                                                    ['blue', 'r'])
+    assert mcolors.to_hex("C0") == '#0000ff'
+    assert mcolors.to_hex("C1") == '#ff0000'
+
+    matplotlib.rcParams['axes.prop_cycle'] = cycler('color',
+                                                    ['xkcd:blue', 'r'])
+    assert mcolors.to_hex("C0") == '#0343df'
+    assert mcolors.to_hex("C1") == '#ff0000'
+
+    matplotlib.rcParams['axes.prop_cycle'] = cycler('color', ['8e4585', 'r'])
+
+    assert mcolors.to_hex("C0") == '#8e4585'
+    # if '8e4585' gets parsed as a float before it gets detected as a hex
+    # colour it will be interpreted as a very large number.
+    # this mustn't happen.
+    assert mcolors.to_rgb("C0")[0] != np.inf
+
+
+def test_conversions():
+    # to_rgba_array("none") returns a (0, 4) array.
+    assert_array_equal(mcolors.to_rgba_array("none"), np.zeros((0, 4)))
+    # a list of grayscale levels, not a single color.
+    assert_array_equal(
+        mcolors.to_rgba_array([".2", ".5", ".8"]),
+        np.vstack([mcolors.to_rgba(c) for c in [".2", ".5", ".8"]]))
+    # alpha is properly set.
+    assert_equal(mcolors.to_rgba((1, 1, 1), .5), (1, 1, 1, .5))
+    assert_equal(mcolors.to_rgba(".1", .5), (.1, .1, .1, .5))
+    # builtin round differs between py2 and py3.
+    assert_equal(mcolors.to_hex((.7, .7, .7)), "#b2b2b2")
+    # hex roundtrip.
+    hex_color = "#1234abcd"
+    assert_equal(mcolors.to_hex(mcolors.to_rgba(hex_color), keep_alpha=True),
+                 hex_color)
+
+
+def test_grey_gray():
+    color_mapping = mcolors._colors_full_map
+    for k in color_mapping.keys():
+        if 'grey' in k:
+            assert color_mapping[k] == color_mapping[k.replace('grey', 'gray')]
+        if 'gray' in k:
+            assert color_mapping[k] == color_mapping[k.replace('gray', 'grey')]
+
+
+def test_tableau_order():
+    dflt_cycle = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728',
+                  '#9467bd', '#8c564b', '#e377c2', '#7f7f7f',
+                  '#bcbd22', '#17becf']
+
+    assert list(mcolors.TABLEAU_COLORS.values()) == dflt_cycle
+
+
 if __name__ == '__main__':
     import nose
     nose.runmodule(argv=['-s', '--with-doctest'], exit=False)
