@@ -1,3 +1,12 @@
+"""
+=====
+Decay
+=====
+
+This example showcases a sinusoidal decay animation.
+"""
+
+
 import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib.animation as animation
