@@ -1,6 +1,9 @@
-#!/usr/local/bin/python3.6
 """
+=================
 An animated image
+=================
+
+This example demonstrates how to animate an image.
 """
 import numpy as np
 import matplotlib.pyplot as plt
@@ -15,7 +18,7 @@ def f(x, y):
 x = np.linspace(0, 2 * np.pi, 120)
 y = np.linspace(0, 2 * np.pi, 100).reshape(-1, 1)
 
-im = plt.imshow(f(x, y), cmap=plt.get_cmap('viridis'), animated=True)
+im = plt.imshow(f(x, y), animated=True)
 
 
 def updatefig(*args):
