@@ -1,6 +1,9 @@
-#!/usr/local/bin/python3.6
 # -*- noplot -*-
 """
+=============================
+The object-oriented interface
+=============================
+
 A pure OO (look Ma, no pylab!) example using the agg backend
 """
 from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
