@@ -1,5 +1,8 @@
-#!/usr/local/bin/python3.6
 """
+================
+Date tick labels
+================
+
 Show how to make date plots in matplotlib using date tick locators and
 formatters.  See major_minor_demo1.py for more information on
 controlling major and minor ticks
@@ -28,7 +31,7 @@ yearsFmt = mdates.DateFormatter('%Y')
 datafile = cbook.get_sample_data('goog.npy')
 try:
     # Python3 cannot load python2 .npy files with datetime(object) arrays
-    # unless the encoding is set to bytes. Hovever this option was
+    # unless the encoding is set to bytes. However this option was
     # not added until numpy 1.10 so this example will only work with
     # python 2 or with numpy 1.10 and later.
     r = np.load(datafile, encoding='bytes').view(np.recarray)
