@@ -1,10 +1,8 @@
-#!/usr/local/bin/python3.6
-
 from __future__ import print_function
 """
 This is a demo script to show you how to use all the properties of an
 FT2Font object.  These describe global font properties.  For
-individual character metrices, use the Glyp object, as returned by
+individual character metrics, use the Glyph object, as returned by
 load_char
 """
 import matplotlib
@@ -12,7 +10,7 @@ import matplotlib.ft2font as ft
 
 
 #fname = '/usr/local/share/matplotlib/VeraIt.ttf'
-fname = matplotlib.get_data_path() + '/fonts/ttf/VeraIt.ttf'
+fname = matplotlib.get_data_path() + '/fonts/ttf/DejaVuSans-Oblique.ttf'
 #fname = '/usr/local/share/matplotlib/cmr10.ttf'
 
 font = ft.FT2Font(fname)
@@ -20,7 +18,7 @@ font = ft.FT2Font(fname)
 print('Num faces   :', font.num_faces)        # number of faces in file
 print('Num glyphs  :', font.num_glyphs)       # number of glyphs in the face
 print('Family name :', font.family_name)      # face family name
-print('Syle name   :', font.style_name)       # face syle name
+print('Style name  :', font.style_name)       # face style name
 print('PS name     :', font.postscript_name)  # the postscript name
 print('Num fixed   :', font.num_fixed_sizes)  # number of embedded bitmap in face
 
@@ -63,5 +61,4 @@ for style in ('Italic',
 
 print(dir(font))
 
-cmap = font.get_charmap()
 print(font.get_kerning)
