@@ -11,7 +11,7 @@ import matplotlib.pyplot as plt
 from mpl_toolkits.mplot3d import Axes3D
 
 
-def lorenz(x, y, z, s=10, r=28, b=2.667) :
+def lorenz(x, y, z, s=10, r=28, b=2.667):
     x_dot = s*(y - x)
     y_dot = r*x - y - x*z
     z_dot = x*y - b*z
@@ -30,7 +30,7 @@ zs = np.empty((stepCnt + 1,))
 xs[0], ys[0], zs[0] = (0., 1., 1.05)
 
 # Stepping through "time".
-for i in range(stepCnt) :
+for i in range(stepCnt):
     # Derivatives of the X, Y, Z state
     x_dot, y_dot, z_dot = lorenz(xs[i], ys[i], zs[i])
     xs[i + 1] = xs[i] + (x_dot * dt)
@@ -40,11 +40,10 @@ for i in range(stepCnt) :
 fig = plt.figure()
 ax = fig.gca(projection='3d')
 
-ax.plot(xs, ys, zs)
+ax.plot(xs, ys, zs, lw=0.5)
 ax.set_xlabel("X Axis")
 ax.set_ylabel("Y Axis")
 ax.set_zlabel("Z Axis")
 ax.set_title("Lorenz Attractor")
 
 plt.show()
-
