@@ -1,6 +1,6 @@
 from __future__ import print_function
 import numpy
-from matplotlib.pyplot import figure, show
+import matplotlib.pyplot as plt
 
 
 class IndexTracker(object):
@@ -21,7 +21,6 @@ class IndexTracker(object):
             self.ind = numpy.clip(self.ind + 1, 0, self.slices - 1)
         else:
             self.ind = numpy.clip(self.ind - 1, 0, self.slices - 1)
-
         self.update()
 
     def update(self):
@@ -30,8 +29,7 @@ class IndexTracker(object):
         self.im.axes.figure.canvas.draw()
 
 
-fig = figure()
-ax = fig.add_subplot(111)
+fig, ax = plt.subplots(1, 1)
 
 X = numpy.random.rand(20, 20, 40)
 
@@ -39,4 +37,4 @@ tracker = IndexTracker(ax, X)
 
 
 fig.canvas.mpl_connect('scroll_event', tracker.onscroll)
-show()
+plt.show()
