@@ -2,7 +2,7 @@ import matplotlib.pyplot as plt
 import numpy as np
 
 t = np.arange(0.0, 2.0, 0.01)
-s = np.sin(2*np.pi*t)
+s = 1 + np.sin(2*np.pi*t)
 plt.plot(t, s)
 
 plt.xlabel('time (s)')
