@@ -1,5 +1,3 @@
-#!/usr/local/bin/python3.6
-
 import matplotlib.pyplot as plt
 import numpy as np
 
@@ -26,4 +24,6 @@ plt.yscale('symlog', linthreshy=0.015)
 plt.grid(True)
 plt.ylabel('symlog both')
 
+plt.subplots_adjust(hspace=0.5, left=0.2)
+
 plt.show()
