@@ -1,4 +1,3 @@
-#!/usr/local/bin/python3.6
 # -*- noplot -*-
 """
 This example shows how to use the agg backend directly to create
@@ -37,7 +36,7 @@ def make_fig():
     # make a translucent scatter collection
     x = np.random.rand(100)
     y = np.random.rand(100)
-    area = np.pi * (10 * np.random.rand(100)) ** 2  # 0 to 10 point radiuses
+    area = np.pi * (10 * np.random.rand(100)) ** 2  # 0 to 10 point radii
     c = ax.scatter(x, y, area)
     c.set_alpha(0.5)
 
