@@ -1,4 +1,13 @@
-#!/usr/local/bin/python3.6
+"""
+=========================
+Group barchart with units
+=========================
+
+This is the same example as
+<a href='http://matplotlib.org/examples/api/barchart_demo.html'>
+the barchart demo</a> in centimeters.
+"""
+
 import numpy as np
 from basic_units import cm, inch
 import matplotlib.pyplot as plt
@@ -17,15 +26,15 @@ p1 = ax.bar(ind, menMeans, width, color='r', bottom=0*cm, yerr=menStd)
 
 womenMeans = (145*cm, 149*cm, 172*cm, 165*cm, 200*cm)
 womenStd = (30*cm, 25*cm, 20*cm, 31*cm, 22*cm)
-p2 = ax.bar(ind + width, womenMeans, width, color='y', bottom=0*cm, yerr=womenStd)
+p2 = ax.bar(ind + width, womenMeans, width,
+            color='y', bottom=0*cm, yerr=womenStd)
 
 ax.set_title('Scores by group and gender')
-ax.set_xticks(ind + width)
+ax.set_xticks(ind + width / 2)
 ax.set_xticklabels(('G1', 'G2', 'G3', 'G4', 'G5'))
 
 ax.legend((p1[0], p2[0]), ('Men', 'Women'))
 ax.yaxis.set_units(inch)
 ax.autoscale_view()
 
-#plt.savefig('barchart_demo')
 plt.show()
